import 'dart:async';
import 'package:flutter/material.dart';
import 'package:tivio_sdk/player/tivio_player_input_source.dart';
import 'package:better_player/better_player.dart';
import 'better_player_material_progress_bar.dart';
import 'svg_button.dart';
import 'better_player_ui_press_controls.dart';
import '../tivio_player_colors.dart';

// TD make this independent from BetterPlayerController
class BetterPlayerControlUi extends StatelessWidget {
  final BetterPlayerUiPressControls controls;
  final BetterPlayerController controller;
  final TivioPlayerColors tivioPlayerColors;

  final bool isPlaying;
  final bool isInFullscreen;

  final Duration? duration;
  final Duration? progress;

  BetterPlayerControlUi({
    required this.controller,
    required this.controls,
    required this.tivioPlayerColors,
    this.isPlaying = true,
    this.isInFullscreen = true,
    this.duration,
    this.progress,
  });

  String formatNumber(int number) {
    return (number > 9) ? number.toString() : '0$number';
  }

  String createCounterLabel() {
    int restOfSeconds = (duration != null && progress != null) ? duration!.inSeconds - progress!.inSeconds : 0;

    Duration restAsDuration = Duration(seconds: restOfSeconds);

    return '-${formatNumber(restAsDuration.inMinutes)}:${formatNumber(restAsDuration.inSeconds % 60)}';
  }

  @override
  Widget build(BuildContext context) {
    String counterLabel = createCounterLabel();

    return Padding(
      padding: EdgeInsets.all(0),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.end,
        children: [
          Flexible(
            fit: FlexFit.tight,
            flex: 1,
            child: Padding(
              padding: EdgeInsets.only(top: 80),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  SvgButton(
                    iconName: 'backward',
                    onTap: () => controls.onBackwardPressed(),
                    width: 60,
                    height: 60,
                    padding: 20,
                  ),
                  SizedBox(width: 30),
                  SvgButton(
                    iconName: (isPlaying) ? 'pause' : 'play',
                    onTap: () {
                      if (isPlaying) {
                        controls.onPausePressed();
                      } else {
                        controls.onPlayPressed();
                      }
                    },
                    width: 70,
                    height: 70,
                    padding: 10,
                  ),
                  SizedBox(width: 30),
                  SvgButton(
                    iconName: 'forward',
                    onTap: () => controls.onForwardPressed(),
                    width: 60,
                    height: 60,
                    padding: 20,
                  ),
                ],
              ),
            ),
          ),
          Padding(
            padding: EdgeInsets.only(left: 26, right: 26, bottom: 13),
            child: Column(
              children: [
                Row(
                  mainAxisAlignment: MainAxisAlignment.end,
                  children: [
                    Padding(
                      padding: EdgeInsets.only(bottom: 0),
                      child: SvgButton(
                        iconName: (isInFullscreen) ? 'fullscreen-exit' : 'fullscreen-enter',
                        onTap: () {
                          if (isInFullscreen) {
                            controls.onExitFullscreenPressed();
                          } else {
                            controls.onEnterFullscreenPressed();
                          }
                        },
                        width: (isInFullscreen) ? 40 : 25,
                      ),
                    ),
                  ],
                ),
                Row(
                  children: [
                    Expanded(
                      child: BetterPlayerMaterialVideoProgressBar(
                        controller.videoPlayerController,
                        controller,
                        colors: BetterPlayerProgressColors(
                          playedColor: tivioPlayerColors.progressBarPlayed,
                          bufferedColor: Color.fromRGBO(255, 255, 255, 0.2),
                          backgroundColor: Color.fromRGBO(255, 255, 255, 0.2),
                        ),
                      ),
                    ),
                    Padding(
                      padding: EdgeInsets.only(left: 10, top: 10),
                      child: Text(
                        counterLabel,
                        style: TextStyle(
                          fontSize: 18,
                          color: Color.fromRGBO(255, 255, 255, 0.7),
                        ),
                      ),
                    ),
                  ],
                ),
              ],
            ),
          )
        ],
      ),
    );
  }
}

class BetterPlayerControl extends StatefulWidget {
  final BetterPlayerUiPressControls controls;
  final BetterPlayerController controller;
  final TivioPlayerColors tivioPlayerColors;

  StreamController<BetterPlayerEvent> eventPlayerStream;
  TivioPlayerVodSource source;

  BetterPlayerControl({
    required this.controls,
    required this.controller,
    required this.tivioPlayerColors,
    required this.eventPlayerStream,
    required this.source,
  });

  @override
  createState() => _BetterPlayerControlState();
}

class _BetterPlayerControlState extends State<BetterPlayerControl> with SingleTickerProviderStateMixin {
  Duration? progress = Duration();
  StreamSubscription? subscription;
  bool isPlaying = true;
  bool isInFullscreen = true;
  Timer? showUiTimer;
  late AnimationController controller;

  @override
  void initState() {
    super.initState();

    if (subscription != null) {
      subscription!.cancel();
    }

    subscription = widget.eventPlayerStream.stream.listen((event) {
      switch (event.betterPlayerEventType) {
        case BetterPlayerEventType.play:
          hideUiAfterDelay();
          setState(() {
            isPlaying = true;
          });
          break;
        case BetterPlayerEventType.pause:
          showUiAndCancelDelay();
          setState(() {
            isPlaying = false;
          });
          break;
        case BetterPlayerEventType.progress:
          setState(() {
            progress = event.parameters!['progress'];
          });
          break;
        case BetterPlayerEventType.finished:
          // fix missing last tick
          setState(() {
            progress = widget.controller.videoPlayerController!.value.duration;
          });
          break;
        case BetterPlayerEventType.hideFullscreen:
          setState(() {
            isInFullscreen = false;
          });
          break;
        case BetterPlayerEventType.openFullscreen:
          setState(() {
            isInFullscreen = true;
          });
          break;
        default:
          break;
      }
    });

    hideUiAfterDelay();
    setupAnimation();
  }

  showUi() {
    controller.animateTo(1);
  }

  hideUi() {
    controller.animateTo(0);
  }

  hideUiAfterDelay() {
    if (showUiTimer != null) {
      showUiTimer!.cancel();
    }
    showUiTimer = Timer(Duration(seconds: 3), () {
      hideUi();
    });
  }

  showUiAndCancelDelay() {
    if (showUiTimer != null) {
      showUiTimer!.cancel();
    }
    showUi();
  }

  showUiAndHideAfterDelay() {
    showUi();
    hideUiAfterDelay();
  }

  setupAnimation() {
    controller = AnimationController(
      value: 1,
      duration: Duration(milliseconds: 300),
      vsync: this,
    );
  }

  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: () => showUiAndHideAfterDelay(),
      child: FadeTransition(
        opacity: controller,
        child: Container(
          width: 100.0,
          height: 100.0,
          child: BetterPlayerControlUi(
            controller: widget.controller,
            controls: widget.controls,
            tivioPlayerColors: widget.tivioPlayerColors,
            isPlaying: isPlaying,
            progress: progress,
            duration: widget.controller.videoPlayerController!.value.duration,
            isInFullscreen: isInFullscreen,
          ),
        ),
      ),
    );
  }

  @override
  void dispose() {
    if (subscription != null) {
      subscription!.cancel();
    }
    if (showUiTimer != null) {
      showUiTimer!.cancel();
    }
    super.dispose();
  }
}
