import 'dart:async';
import 'package:better_player/better_player.dart';
import 'package:flutter/material.dart';
import 'tivio_player_loader.dart';
import 'tivio_player_colors.dart';
import 'controls/better_player_ad_controls.dart';
import 'controls/better_player_control.dart';
import 'controls/better_player_ui_press_controls.dart';
import '../tivio.dart';
import '../player/tivio_player_input_source.dart';
import '../player/tivio_player_wrapper.dart';
import '../player/tivio_player_controller.dart';
import '../logger.dart';
import '../enums/tivio_player_state.dart';

class TivioPlayer extends StatefulWidget {
  final String playerId;
  final Tivio tivio;
  final TivioDataVideo? tivioVideoSource;
  final TivioPlayerController tivioPlayerController;
  final TivioPlayerColors tivioPlayerColors;
  final bool startInFullscreen;

  TivioPlayer({
    required this.playerId,
    required this.tivio,
    this.tivioVideoSource,
    this.startInFullscreen = false,
    tivioPlayerController,
    tivioPlayerColors,
  })  : tivioPlayerController = tivioPlayerController ?? TivioPlayerController(),
        tivioPlayerColors = tivioPlayerColors ?? TivioPlayerColors();

  @override
  _TivioPlayerState createState() => _TivioPlayerState();
}

class _TivioPlayerState extends State<TivioPlayer> {
  final Logger logger = Logger();
  bool isBetterPlayerInitialized = false;
  late TivioPlayerWrapper tivioPlayerWrapper;
  BetterPlayerController? betterPlayerController;
  StreamController<BetterPlayerEvent>? playerEventStream;
  int? seekToAfterInitialization;
  Map<String, BetterPlayerDataSource> sources = Map();
  TivioPlayerState playerState = TivioPlayerState.idle;

  @override
  void initState() {
    super.initState();
    synchronousInitState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      color: Colors.black,
      child: (isBetterPlayerInitialized)
          ? BetterPlayer(controller: betterPlayerController!)
          : TivioPlayerLoader(fullscreen: widget.startInFullscreen),
    );
  }

  @override
  void dispose() {
    if (playerEventStream != null) {
      playerEventStream!.close();
    }
    betterPlayerController = null;
    super.dispose();
  }

  /// Public

  openAndPlay(TivioDataVideo tivioDataVideo) {
    if (tivioDataVideo.isPlaylist) {
      throw Exception('Playlists are currently unsupported');
    }

    tivioPlayerWrapper.openAndPlay(TivioPlayerVodSource(
      id: tivioDataVideo.id,
      uri: tivioDataVideo.url,
      name: tivioDataVideo.name,
      description: '',
    ));
  }

  /// Player wrapper integration

  playerPlay(payload) {
    betterPlayerController!.play();
  }

  playerPause(payload) {
    betterPlayerController!.pause();
  }

  playerSeekTo(Duration time) {
    betterPlayerController!.seekTo(time);
  }

  playerOpenAndPlay(TivioPlayerInputSource source, {int? currentTime}) async {
    logger.verbose('Open and play source id: ${source.id}, type: ${source.type}, will seek to: $currentTime');

    seekToAfterInitialization = currentTime;

    bool isSourceAd = (source.type == TivioPlayerWrapperInputSourceType.Advertisement);
    setupBetterPlayer(source, isSourceAd);
  }

  /// Private

  skipAd() {
    tivioPlayerWrapper.skipAd();
  }

  synchronousInitState() async {
    await initTivioPlayerWrapper();
    initTivioPlayerWrapperController();

    if (this.widget.tivioVideoSource != null) {
      openAndPlay(this.widget.tivioVideoSource!);
    }
  }

  initTivioPlayerWrapper() async {
    tivioPlayerWrapper = await widget.tivio.getPlayer(widget.playerId);

    tivioPlayerWrapper.register(TivioPlayerWrapperController(
      playerPlay,
      playerPause,
      playerSeekTo,
      playerOpenAndPlay,
    ));
  }

  initTivioPlayerWrapperController() {
    widget.tivioPlayerController.play = tivioPlayerWrapper.play;
    widget.tivioPlayerController.pause = tivioPlayerWrapper.pause;
    widget.tivioPlayerController.openAndPlay = this.openAndPlay;
    widget.tivioPlayerController.seekTo = tivioPlayerWrapper.seekTo;
  }

  setupBetterPlayer(TivioPlayerInputSource tivioSource, bool isSourceAd) {
    if (betterPlayerController != null) {
      betterPlayerController!.pause();
      betterPlayerController!.removeEventsListener(this.handleBetterPlayerEvent);
    }

    if (playerEventStream != null) {
      playerEventStream!.close();
    }

    var newPlayerEventStream = StreamController<BetterPlayerEvent>.broadcast();
    var config = BetterPlayerConfiguration(
      aspectRatio: 16 / 9,
      fit: BoxFit.fill,
      autoPlay: false,
      autoDispose: true,
      fullScreenByDefault: widget.startInFullscreen,
      controlsConfiguration: (isSourceAd)
          ? BetterPlayerControlsConfiguration(
              playerTheme: BetterPlayerTheme.custom,
              customControlsBuilder: (controller) => BetterPlayerAdControls(
                eventPlayerStream: newPlayerEventStream,
                source: tivioSource as TivioPlayerAdSource,
                onClick: () => skipAd(),
              ),
            )
          : BetterPlayerControlsConfiguration(
              playerTheme: BetterPlayerTheme.custom,
              customControlsBuilder: (controller) => BetterPlayerControl(
                controller: controller,
                eventPlayerStream: newPlayerEventStream,
                tivioPlayerColors: widget.tivioPlayerColors,
                source: tivioSource as TivioPlayerVodSource,
                controls: BetterPlayerUiPressControls(
                  onPlayPressed: () => widget.tivioPlayerController.play(),
                  onPausePressed: () => widget.tivioPlayerController.pause(),
                  onForwardPressed: () async {
                    Duration position = await (controller.videoPlayerController!.position as FutureOr<Duration>);
                    widget.tivioPlayerController.seekTo(Duration(seconds: position.inSeconds + 15));
                  },
                  onBackwardPressed: () async {
                    Duration position = await (controller.videoPlayerController!.position as FutureOr<Duration>);
                    widget.tivioPlayerController.seekTo(Duration(seconds: position.inSeconds - 15));
                  },
                  onEnterFullscreenPressed: () => controller.enterFullScreen(),
                  onExitFullscreenPressed: () => controller.exitFullScreen(),
                  onSeekingStarted: () => widget.tivioPlayerController.pause(),
                  onSeekingEnded: (value) {
                    Duration sourceDuration = controller.videoPlayerController!.value.duration!;
                    int seekToSecond = (sourceDuration.inSeconds * value).ceil();
                    widget.tivioPlayerController.seekTo(Duration(seconds: seekToSecond));
                  },
                ),
              ),
            ),
    );

    var source = createPlayerSource(tivioSource);
    var controller = BetterPlayerController(config);

    setState(() {
      betterPlayerController = controller;
      betterPlayerController!.setupDataSource(source!);
      betterPlayerController!.addEventsListener(this.handleBetterPlayerEvent);
      betterPlayerController!.pause();

      isBetterPlayerInitialized = true;
      playerEventStream = newPlayerEventStream;
    });
  }

  BetterPlayerDataSource? createPlayerSource(TivioPlayerInputSource tivioPlayerInputSource) {
    switch (tivioPlayerInputSource.type) {
      case TivioPlayerWrapperInputSourceType.Vod:
        if (tivioPlayerInputSource.id != null && sources[tivioPlayerInputSource.id!] != null) {
          return sources[tivioPlayerInputSource.id!];
        }

        var source = BetterPlayerDataSource(
          BetterPlayerDataSourceType.network,
          tivioPlayerInputSource.uri!,
          cacheConfiguration: BetterPlayerCacheConfiguration(
            useCache: true,
            preCacheSize: 100 * 1024 * 1024,
            maxCacheSize: 100 * 1024 * 1024,
            maxCacheFileSize: 100 * 1024 * 1024,
          ),
        );

        if (tivioPlayerInputSource.id != null) {
          sources[tivioPlayerInputSource.id!] = source;
        }
        return source;
      case TivioPlayerWrapperInputSourceType.Advertisement:
        return BetterPlayerDataSource(
          BetterPlayerDataSourceType.network,
          tivioPlayerInputSource.uri!,
        );
      default:
        throw Exception('Unsupported type ${tivioPlayerInputSource.type} in createPlayerSource');
    }
  }

  handleBetterPlayerEvent(BetterPlayerEvent event) {
    if (playerEventStream != null) {
      playerEventStream!.add(event);
    }

    switch (event.betterPlayerEventType) {
      case BetterPlayerEventType.initialized:
        if (seekToAfterInitialization != null) {
          playerSeekTo(Duration(milliseconds: seekToAfterInitialization!));
        }
        if (betterPlayerController != null) {
          betterPlayerController!.play();
        } else {
          logger.error('Unable to play video. Player is not initialized.');
        }
        break;
      case BetterPlayerEventType.play:
        tivioPlayerWrapper.onStateChanged(TivioPlayerState.playing);
        break;
      case BetterPlayerEventType.pause:
        tivioPlayerWrapper.onStateChanged(TivioPlayerState.paused);
        break;
      case BetterPlayerEventType.progress:
        tivioPlayerWrapper.onTimeChanged(event.parameters!['progress']);
        break;
      case BetterPlayerEventType.seekTo:
        tivioPlayerWrapper.onTimeChanged(event.parameters!['duration']);
        break;
      case BetterPlayerEventType.finished:
        tivioPlayerWrapper.onPlaybackEnded();
        // if current source = vod -> call onEnded
        // if current souurce = ad & is last -> call onEnded
        break;
      default:
        break;
    }
  }
}
